(function() {
  var http, url;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  http = require('http');
  url = require('url');
  exports.setupESI = function() {
    var handleESI;
    return handleESI = function(req, res, next) {
      var end, endWait, write;
      endWait = false;
      write = res.write;
      res.write = function(chunk, encoding) {
        var content_type, esiProcessor, host, uri;
        res.write = write;
        content_type = (res._headers != null) && (res._headers['content-type'] != null) ? res._headers['content-type'] : 'text/plain';
        if (content_type.indexOf('text') !== -1) {
          console.log("\nres.write\nrequest.url: " + req.url + " content-type: " + content_type);
          endWait = true;
          esiProcessor = new exports.processESI(chunk, function(err, chunk) {
            var _ref;
            res.writeHead((_ref = res.statusCode) != null ? _ref : 200, {
              'Content-Length': chunk.length,
              'Content-Type': content_type
            });
            endWait = false;
            return res.write(chunk, encoding);
          });
          host = req.headers['host'].indexOf('http://') === 0 ? req.headers['host'] : 'http://' + req.headers['host'];
          uri = url.parse(host);
          esiProcessor.hostname = uri.hostname;
          esiProcessor.port = uri.port;
          return esiProcessor.execute();
        } else {
          return res.write(chunk, encoding);
        }
      };
      end = res.end;
      res.end = function(data, encoding) {
        res.end = end;
        if (data) {
          res.write(data, encoding);
        }
        if (endWait !== true) {
          return res.end();
        }
      };
      return next();
    };
  };
  exports.processESI = function(data, callback) {
    this.data = data.toString();
    this.callback = callback;
    this.hostname = 'localhost';
    this.port = 5678;
    this.execute = function() {
      var esi_tags, k, _results;
      console.log("Searching for ESIs...");
      esi_tags = this.searchESI(this.data);
      console.log("Found " + esi_tags.length + " ESIs");
      if (esi_tags.length !== 0) {
        _results = [];
        for (k in esi_tags) {
          console.log("Fetching ESI '" + esi_tags[k] + "'");
          _results.push(this.fetchESI(esi_tags[k], __bind(function(src, status, body) {
            var remaining;
            console.log("Fetch ESI '" + src + "' returned status " + status);
            if (status !== 200) {
              body = '';
            }
            console.log("ESI tag subsitution for '" + src + "'");
            remaining = this.subESI(src, body);
            console.log(remaining + " remaining ESIs");
            if (remaining === 0) {
              console.log("Display Data\n");
              return this.callback(false, this.data);
            }
          }, this)));
        }
        return _results;
      } else {
        console.log("Display Data\n");
        return this.callback(false, this.data);
      }
    };
    this.searchESI = function(data) {
      var esi_tags, k, match, re, _ref;
      re = new RegExp('<esi[^>]* src=("[^>"]+"|\'[^>\']+\')[^>]*>(</esi[^>]*>)?', 'gi');
      esi_tags = data.match(re);
      for (k in esi_tags) {
        re = new RegExp(' src=("(.*?)"|\'(.*?)\')');
        match = esi_tags[k].match(re);
        esi_tags[k] = (_ref = match[2]) != null ? _ref : match[3];
      }
      return esi_tags != null ? esi_tags : [];
    };
    this.fetchESI = function(src, callback) {
      var options, uri, _ref, _ref2, _ref3, _ref4, _ref5;
      uri = url.parse(src);
      options = {
        host: (_ref = uri.hostname) != null ? _ref : this.hostname,
        port: (_ref2 = uri.port) != null ? _ref2 : (uri.protocol != null ? 80 : this.port),
        path: ((_ref3 = uri.pathname) != null ? _ref3 : '/') + ((_ref4 = uri.search) != null ? _ref4 : '') + ((_ref5 = uri.hash) != null ? _ref5 : '')
      };
      return http.get(options, function(res) {
        var body;
        body = '';
        res.on('data', __bind(function(chunk) {
          return body += chunk.toString();
        }, this));
        return res.on('end', __bind(function() {
          return callback(src, res.statusCode, body);
        }, this));
      }).on('error', function(e) {
        return callback(src, 500, e.message);
      });
    };
    this.subESI = function(src, body) {
      var re;
      re = new RegExp('<esi[^>]* src=("' + RegExp.escape(src) + '"|\'' + RegExp.escape(src) + '\')[^>]*>(</esi[^>]*>)?', 'gi');
      this.data = this.data.replace(re, body);
      return this.searchESI(this.data).length;
    };
    RegExp.escape = function(text) {
      return text.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
    };
  };
}).call(this);
