connect = require 'connect'
connectESI = require 'connect-esi'
assert = require 'assert'

data = 'Start. Test data one: <esi:include src="/one.html" /> Test data two: <esi:include src="/two.html"></esi:include> Test data three: <esi:include src=\'/three.html\' /> End.'
esi = new connectESI.processESI data, (err, data) ->
	return
	
module.exports =
	"processESI" : ->	
		assert.equal esi.data, data, "\nprocessESI Expected: #{data}.\nActual: #{esi.data}"		

	"searchESI" : ->	
		esi_arr = esi.searchESI(data)
		assert.equal esi_arr, "/one.html,/two.html,/three.html", "searchESI Expected: #{expected}. Actual: #{esi_arr}"
		assert.equal esi_arr.length, expected=3, "searchESI Expected: #{expected}. Actual: #{esi_arr.length}"

	"fetchESI" : ->				
		esi.fetchESI 'http://www.canada.com/_tech/steve_v/hello.js', (src, status, body) ->
			assert.equal status, expected=200, "fetchESI - HTTP status code. Expected: #{expected}. Actual: #{status}."
	
		esi.fetchESI 'http://www.canada.com/_tech/steve_v/goodbye.js', (src, status, body) ->
			assert.equal status, expected=302, "fetchESI - HTTP status code. Expected: #{expected}. Actual: #{status}."	
	
		esi.fetchESI 'http://www2.canada.com/_tech/steve_v/goodbye.js', (src, status, body) ->
			assert.equal status, expected=404, "fetchESI - HTTP status code. Expected: #{expected}. Actual: #{status}."
	
		esi.fetchESI 'thiswillfail/fail.html', (src, status, body) ->
			assert.equal status, expected=500, "fetchESI - HTTP status code. Expected: #{expected}. Actual: #{status}."

	"subESI" : ->			
		cnt = esi.subESI('', '')
		assert.equal cnt, expected=3, "subESI Expected: #{expected}. Actual: #{cnt}."

		cnt = esi.subESI('/four.html', 'ESI #4')
		assert.equal cnt, expected=3, "subESI Expected: #{expected}. Actual: #{cnt}."
		
		cnt = esi.subESI('/three.html', 'ESI #3')
		assert.equal cnt, expected=2, "subESI Expected: #{expected}. Actual: #{cnt}."		

		cnt = esi.subESI('/one.html', 'ESI #1')
		assert.equal cnt, expected=1, "subESI Expected: #{expected}. Actual: #{cnt}."

		cnt = esi.subESI('/two.html', 'ESI #2')
		assert.equal cnt, expected=0, "subESI Expected: #{expected}. Actual: #{cnt}."
		
		expected = 'Start. Test data one: ESI #1 Test data two: ESI #2 Test data three: ESI #3 End.'
		assert.equal esi.data, expected, "\nsubESI Expected: #{expected}.\nActual: #{esi.data}"

	"execute request (server stopped)" : ->	
		esi = new connectESI.processESI data, (err, data) ->
			assert.ok not err, "execute had the following error: #{err}"
			expected = 'Start. Test data one:  Test data two:  Test data three:  End.'
			assert.equal data, expected, "\nexecute Expected: #{expected}.\nActual: #{data}."	
		esi.port = 1234
		esi.execute()

	"execute request (server started)" : ->		
		# create server for testing		
		server = connect.createServer()
		server.use connectESI.setupESI()
		server.use (req, res, next) ->
			body = ''
			switch req.url
				when '/one.html' then body = 'ESI #1'
				when '/two.html' then body = 'ESI #2'
				when '/three.html' then body = 'ESI #3'
				else
					body = 'ESI ' + req.url
					res.statusCode = 404
			res.write body
			res.end()
		server.listen 5678

		esi = new connectESI.processESI data, (err, data) ->
			server.close()
			assert.ok not err, "execute had the following error: #{err}"
			expected = 'Start. Test data one: ESI #1 Test data two: ESI #2 Test data three: ESI #3 End.'
			assert.equal data, expected, "\nexecute Expected: #{expected}.\nActual: #{data}."	
		esi.execute()
		
	"execute request (external)" : ->
		esi = new connectESI.processESI "<esi:include src=\"http://www.canada.com/_tech/steve_v/hello.js\" />", (err, data) ->
			assert.ok not err, "execute had the following error: #{err}"
			expected = "document.write('Hello World!');"
			assert.equal data, expected, "\nexecute Expected: #{expected}.\nActual: #{data}."
		esi.execute()
		