(function() {
  var assert, connect, connectESI, data, esi;
  connect = require('connect');
  connectESI = require('connect-esi');
  assert = require('assert');
  data = 'Start. Test data one: <esi:include src="/one.html" /> Test data two: <esi:include src="/two.html"></esi:include> Test data three: <esi:include src=\'/three.html\' /> End.';
  esi = new connectESI.processESI(data, function(err, data) {});
  module.exports = {
    "processESI": function() {
      return assert.equal(esi.data, data, "\nprocessESI Expected: " + data + ".\nActual: " + esi.data);
    },
    "searchESI": function() {
      var esi_arr, expected;
      esi_arr = esi.searchESI(data);
      assert.equal(esi_arr, "/one.html,/two.html,/three.html", "searchESI Expected: " + expected + ". Actual: " + esi_arr);
      return assert.equal(esi_arr.length, expected = 3, "searchESI Expected: " + expected + ". Actual: " + esi_arr.length);
    },
    "fetchESI": function() {
      esi.fetchESI('http://www.canada.com/_tech/steve_v/hello.js', function(src, status, body) {
        var expected;
        return assert.equal(status, expected = 200, "fetchESI - HTTP status code. Expected: " + expected + ". Actual: " + status + ".");
      });
      esi.fetchESI('http://www.canada.com/_tech/steve_v/goodbye.js', function(src, status, body) {
        var expected;
        return assert.equal(status, expected = 302, "fetchESI - HTTP status code. Expected: " + expected + ". Actual: " + status + ".");
      });
      esi.fetchESI('http://www2.canada.com/_tech/steve_v/goodbye.js', function(src, status, body) {
        var expected;
        return assert.equal(status, expected = 404, "fetchESI - HTTP status code. Expected: " + expected + ". Actual: " + status + ".");
      });
      return esi.fetchESI('thiswillfail/fail.html', function(src, status, body) {
        var expected;
        return assert.equal(status, expected = 500, "fetchESI - HTTP status code. Expected: " + expected + ". Actual: " + status + ".");
      });
    },
    "subESI": function() {
      var cnt, expected;
      cnt = esi.subESI('', '');
      assert.equal(cnt, expected = 3, "subESI Expected: " + expected + ". Actual: " + cnt + ".");
      cnt = esi.subESI('/four.html', 'ESI #4');
      assert.equal(cnt, expected = 3, "subESI Expected: " + expected + ". Actual: " + cnt + ".");
      cnt = esi.subESI('/three.html', 'ESI #3');
      assert.equal(cnt, expected = 2, "subESI Expected: " + expected + ". Actual: " + cnt + ".");
      cnt = esi.subESI('/one.html', 'ESI #1');
      assert.equal(cnt, expected = 1, "subESI Expected: " + expected + ". Actual: " + cnt + ".");
      cnt = esi.subESI('/two.html', 'ESI #2');
      assert.equal(cnt, expected = 0, "subESI Expected: " + expected + ". Actual: " + cnt + ".");
      expected = 'Start. Test data one: ESI #1 Test data two: ESI #2 Test data three: ESI #3 End.';
      return assert.equal(esi.data, expected, "\nsubESI Expected: " + expected + ".\nActual: " + esi.data);
    },
    "execute request (server stopped)": function() {
      esi = new connectESI.processESI(data, function(err, data) {
        var expected;
        assert.ok(!err, "execute had the following error: " + err);
        expected = 'Start. Test data one:  Test data two:  Test data three:  End.';
        return assert.equal(data, expected, "\nexecute Expected: " + expected + ".\nActual: " + data + ".");
      });
      esi.port = 1234;
      return esi.execute();
    },
    "execute request (server started)": function() {
      var server;
      server = connect.createServer();
      server.use(connectESI.setupESI());
      server.use(function(req, res, next) {
        var body;
        body = '';
        switch (req.url) {
          case '/one.html':
            body = 'ESI #1';
            break;
          case '/two.html':
            body = 'ESI #2';
            break;
          case '/three.html':
            body = 'ESI #3';
            break;
          default:
            body = 'ESI ' + req.url;
            res.statusCode = 404;
        }
        res.write(body);
        return res.end();
      });
      server.listen(5678);
      esi = new connectESI.processESI(data, function(err, data) {
        var expected;
        server.close();
        assert.ok(!err, "execute had the following error: " + err);
        expected = 'Start. Test data one: ESI #1 Test data two: ESI #2 Test data three: ESI #3 End.';
        return assert.equal(data, expected, "\nexecute Expected: " + expected + ".\nActual: " + data + ".");
      });
      return esi.execute();
    },
    "execute request (external)": function() {
      esi = new connectESI.processESI("<esi:include src=\"http://www.canada.com/_tech/steve_v/hello.js\" />", function(err, data) {
        var expected;
        assert.ok(!err, "execute had the following error: " + err);
        expected = "document.write('Hello World!');";
        return assert.equal(data, expected, "\nexecute Expected: " + expected + ".\nActual: " + data + ".");
      });
      return esi.execute();
    }
  };
}).call(this);
