http = require 'http'
url = require 'url'

exports.setupESI = ->
	handleESI = (req, res, next) ->
		endWait = false		

		# MODIFY res.write
		write = res.write  # store the original function		
		res.write = (chunk, encoding) -> # wrap write to hook into the exit path through the layers
			res.write = write # put the original back
			content_type = 	if res._headers? and res._headers['content-type']? then	res._headers['content-type'] else 'text/plain' # if no content-type is provided, set to 'text/plain'
			unless content_type.indexOf('text') == -1 # only process ESIs if response data contains text
				console.log "\nres.write\nrequest.url: " + req.url + " content-type: " + content_type
				endWait = true
				esiProcessor = new exports.processESI chunk, (err, chunk) ->
					#console.log "processESI callback"
					res.writeHead (res.statusCode ? 200),
					  'Content-Length': chunk.length
					  'Content-Type': content_type
					endWait = false
					res.write chunk, encoding # call the original on callback
					#res.end()
				host = if req.headers['host'].indexOf('http://') == 0 then req.headers['host'] else 'http://'+req.headers['host']
				uri = url.parse(host)
				esiProcessor.hostname = uri.hostname
				esiProcessor.port = uri.port
				esiProcessor.execute()
			else			
				res.write chunk, encoding # call the original				

		# MODIFY res.end
		end = res.end
		res.end = (data, encoding) ->
			res.end = end
			#console.log "\nres.end\n request.url: " + req.url + " wait: " + endWait
			if data
				res.write data, encoding
			unless endWait == true # only end if not waiting (for async)
				res.end()

		next() # pass through to the next layer

exports.processESI = (data, callback) ->
	@data = data.toString()
	@callback = callback
	@hostname = 'localhost'
	@port = 5678

	@execute = ->
		console.log "Searching for ESIs..."
		esi_tags = @searchESI @data
		console.log "Found " + esi_tags.length + " ESIs"
		if (esi_tags.length != 0) # there are ESIs to process
			for k of esi_tags
				console.log "Fetching ESI '" + esi_tags[k] + "'"
				@fetchESI esi_tags[k], (src, status, body) =>
					console.log "Fetch ESI '" + src + "' returned status " + status
					if status != 200 then body = '' # remove error messages if http status is not 200
					console.log "ESI tag subsitution for '" + src + "'"				
					remaining = @subESI src, body
					console.log remaining + " remaining ESIs"
					if remaining == 0 # no remaining ESIs to process, callback
						console.log "Display Data\n"
						@callback false, @data						
		else
			console.log "Display Data\n"
			@callback false, @data	

	@searchESI = (data) ->
		re = new RegExp('<esi[^>]* src=("[^>"]+"|\'[^>\']+\')[^>]*>(</esi[^>]*>)?', 'gi')
		esi_tags = data.match(re)
		for k of esi_tags
			re = new RegExp(' src=("(.*?)"|\'(.*?)\')')
			match = esi_tags[k].match(re)
			esi_tags[k] = match[2] ? match[3] #quotes surrounding src: 2 - double, 3 - single
		return esi_tags ? []

	@fetchESI = (src, callback) ->
		uri = url.parse(src)
		options =
			host: uri.hostname ? @hostname
			port: uri.port ? (if uri.protocol? then 80 else @port)
			path: (uri.pathname ? '/') + (uri.search ? '') + (uri.hash ? '')
		http.get(options, (res) ->
			body = ''
			res.on 'data', (chunk) =>
				body += chunk.toString()
			res.on 'end', =>
				callback src, res.statusCode, body
		).on 'error', (e) ->
			callback src, 500, e.message

	@subESI = (src, body) ->
		re = new RegExp('<esi[^>]* src=("' + RegExp.escape(src) + '"|\'' + RegExp.escape(src) + '\')[^>]*>(</esi[^>]*>)?', 'gi')
		@data = @data.replace(re, body)
		return @searchESI(@data).length # number of ESIs remaining

	RegExp.escape = (text) ->
	  text.replace /[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"

	return